==> haystack-0.5.0%2B2.tar.gz: lib/src/HNum.dart <==
///
/// Copyright (c) 2018, BAS Services & Graphics, LLC.
/// Licensed under the Academic Free License version 3.0
///

library haystack.hnum;

import 'package:intl/intl.dart';

import 'HVal.dart';
import 'str-utils.dart';

/// HNum wraps a 64-bit floating point number and optional unit name.
///
/// See [Project Haystack](http://project-haystack.org/doc/TagModel#tagKinds)
class HNum extends HVal {

  factory HNum (num val, [String unit]) {
    if (val == 0 && unit == null) return ZERO;

    double value;
    if (val != null) value = val.toDouble(); //fixme what if value is null?

    return new HNum._(value, unit);
  }

  /// Singleton value for zero
  static final HNum ZERO = new HNum._(0.0, null);

  /// Singleton value for positive infinity "Inf"
  static final HNum POS_INF = new HNum._(double.INFINITY, null);

  /// Singleton value for negative infinity "-Inf"
  static final HNum NEG_INF = new HNum._(double.NEGATIVE_INFINITY, null);

  /// Singleton value for not-a-number "NaN".
  static final HNum NaN = new HNum._(double.NAN, null);

  HNum._(this.val, this.unit) {
    if (!isUnitName(unit)) throw new ArgumentError("Invalid unit name: $unit");
  }

  /// Double scalar value.
  final double val;

  /// Unit name or null.
  final String unit;

  /// Hash code is based on val, unit.
  @override
  int get hashCode {
    if (_hashcode == null) {
      _hashcode = val.hashCode;
      if (unit != null) _hashcode ^= unit.hashCode;
    }
    return _hashcode;
  }
  int _hashcode;

  /// Equals is based on val, unit (NaN == NaN)
  @override
  bool operator ==(Object that) {
    if (that is! HNum) return false;
    HNum x = that as HNum;
    if (val != null && x.val != null && val.isNaN) return x.val.isNaN;
    if (val != x.val) return false;
    if (unit == null) return x.unit == null;
    if (x.unit == null) return false;
    return unit == x.unit;
  }

  /// Return sort order as negative, 0, or positive
  @override
  int compareTo(Object that) {
    double thatVal = (that as HNum).val;
    if (this.val < thatVal) return -1;
    if (this.val == thatVal) return 0;
    return 1;
  }

  /// Encode as "n:<float> [unit]"
  @override
  String toJson() {
    StringBuffer s = new StringBuffer();
    s.write("n:");
    _encode(s, true);
    return s.toString();
  }

  /// Encode as floating value followed by optional unit string
  @override
  String toZinc() {
    StringBuffer s = new StringBuffer();
    _encode(s);
    return s.toString();
  }

  void _encode(StringBuffer s, [bool spaceBeforeUnit = false]) {
    if (val == double.INFINITY) {
      s.write("INF");
    } else if (val == double.NEGATIVE_INFINITY) {
      s.write("-INF");
    } else if (val.isNaN) {
      s.write("NaN");
    } else {
      //don't let fractions
      if (val.abs() > 1.0)
        s.write(new NumberFormat('#0.####', 'en_US').format(val));
      else
        s.write(val);

      if (unit != null) {
        if (spaceBeforeUnit) s.write(' ');
        s.write(unit);
      }
    }
  }

  /// Get this number as a duration of milliseconds.
  ///
  /// Raise IllegalStateException if the unit is not a duration unit.
  int millis() {
    String u = this.unit;
    if (u == null) u = "null";
    if (u == "ms"  || u == "millisecond") return val.toInt();
    if (u == "s"   || u == "sec")         return (val*1000.0).toInt();
    if (u == "min" || u == "minute")      return (val*1000.0*60.0).toInt();
    if (u == "h"   || u == "hr")          return (val*1000.0*60.0*60.0).toInt();

    throw new FormatException("Invalid duration unit: $u");
  }

  /// Return if the given string is valid unit name or null.
  ///
  /// This method does *not* check that the unit name is part
  /// of the standard unit database.
  static bool isUnitName(String unit) {
    if (unit == null) return true;
    if (unit.length == 0) return false;

    return unit.codeUnits.every((int c) {
      return (c < 128 && !_unitChars[c])? false : true;
    });
  }

  static final List<bool> _unitChars = (){
    List<bool> result = new List<bool>.filled(128, false);
    for (int i=a_CHAR; i<=z_CHAR; ++i) result[i] = true;
    for (int i=A_CHAR; i<=Z_CHAR; ++i) result[i] = true;
    result[UNDERSCORE_CHAR] = true;
    result[DOLLAR_CHAR] = true;
    result[PERCENT_CHAR] = true;
    result[SLASH_CHAR] = true;

    return result;
  }();

}

==> haystack-0.6.0-dev.0.1%2B2.tar.gz: lib/src/HNum.dart <==
///
/// Copyright (c) 2018, BAS Services & Graphics, LLC.
/// Licensed under the Academic Free License version 3.0
///

library haystack.hnum;

import 'package:intl/intl.dart';

import 'HVal.dart';
import 'str-utils.dart';

/// HNum wraps a 64-bit floating point number and optional unit name.
///
/// See [Project Haystack](http://project-haystack.org/doc/TagModel#tagKinds)
class HNum extends HVal {
  factory HNum(num val, [String unit]) {
    if (val == 0 && unit == null) return ZERO;

    double value;
    if (val != null) value = val.toDouble(); //fixme what if value is null?

    return new HNum._(value, unit);
  }

  /// Singleton value for zero
  static final HNum ZERO = new HNum._(0.0, null);

  /// Singleton value for positive infinity "Inf".
  static final HNum POS_INF = new HNum._(double.infinity, null);

  /// Singleton value for negative infinity "-Inf".
  static final HNum NEG_INF = new HNum._(double.negativeInfinity, null);

  /// Singleton value for not-a-number "NaN".
  static final HNum NaN = new HNum._(double.nan, null);

  HNum._(this.val, this.unit) {
    if (!isUnitName(unit)) throw new ArgumentError("Invalid unit name: $unit");
  }

  /// Double scalar value.
  final double val;

  /// Unit name or null.
  final String unit;

  /// Hash code is based on val, unit.
  @override
  int get hashCode {
    if (_hashcode == null) {
      _hashcode = val.hashCode;
      if (unit != null) _hashcode ^= unit.hashCode;
    }
    return _hashcode;
  }

  int _hashcode;

  /// Equals is based on val, unit (NaN == NaN)
  @override
  bool operator ==(Object that) {
    if (that is! HNum) return false;
    HNum x = that as HNum;
    if (val != null && x.val != null && val.isNaN) return x.val.isNaN;
    if (val != x.val) return false;
    if (unit == null) return x.unit == null;
    if (x.unit == null) return false;
    return unit == x.unit;
  }

  /// Return sort order as negative, 0, or positive
  @override
  int compareTo(Object that) {
    double thatVal = (that as HNum).val;
    if (this.val < thatVal) return -1;
    if (this.val == thatVal) return 0;
    return 1;
  }

  /// Encode as "n:<float> [unit]"
  @override
  String toJson() {
    StringBuffer s = new StringBuffer();
    s.write("n:");
    _encode(s, true);
    return s.toString();
  }

  /// Encode as floating value followed by optional unit string
  @override
  String toZinc() {
    StringBuffer s = new StringBuffer();
    _encode(s);
    return s.toString();
  }

  void _encode(StringBuffer s, [bool spaceBeforeUnit = false]) {
    if (val == double.infinity) {
      s.write("INF");
    } else if (val == double.negativeInfinity) {
      s.write("-INF");
    } else if (val.isNaN) {
      s.write("NaN");
    } else {
      //don't let fractions
      if (val.abs() > 1.0)
        s.write(new NumberFormat('#0.####', 'en_US').format(val));
      else
        s.write(val);

      if (unit != null) {
        if (spaceBeforeUnit) s.write(' ');
        s.write(unit);
      }
    }
  }

  /// Get this number as a duration of milliseconds.
  ///
  /// Raise IllegalStateException if the unit is not a duration unit.
  int millis() {
    String u = this.unit;
    if (u == null) u = "null";
    if (u == "ms" || u == "millisecond") return val.toInt();
    if (u == "s" || u == "sec") return (val * 1000.0).toInt();
    if (u == "min" || u == "minute") return (val * 1000.0 * 60.0).toInt();
    if (u == "h" || u == "hr") return (val * 1000.0 * 60.0 * 60.0).toInt();

    throw new FormatException("Invalid duration unit: $u");
  }

  /// Return if the given string is valid unit name or null.
  ///
  /// This method does *not* check that the unit name is part
  /// of the standard unit database.
  static bool isUnitName(String unit) {
    if (unit == null) return true;
    if (unit.length == 0) return false;

    return unit.codeUnits.every((int c) {
      return (c < 128 && !_unitChars[c]) ? false : true;
    });
  }

  static final List<bool> _unitChars = () {
    List<bool> result = new List<bool>.filled(128, false);
    for (int i = a_CHAR; i <= z_CHAR; ++i) result[i] = true;
    for (int i = A_CHAR; i <= Z_CHAR; ++i) result[i] = true;
    result[UNDERSCORE_CHAR] = true;
    result[DOLLAR_CHAR] = true;
    result[PERCENT_CHAR] = true;
    result[SLASH_CHAR] = true;

    return result;
  }();
}
